#include <iostream>
#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"
#include "cargadorConfiguracion.h"
#include "Textura.h"
#include "Torre.h"
#include "Proyectil.h"
#include "Ciudad.h"
#include "Iluminacion.h"
#include <math.h>       /* sin */
#include <map>

#define FPS 60
#define ALTO 640.0
#define ANCHO 480.0
#define FRONT 0.1
#define REAR 200
#define ANGULO 45
#define FACTOR_DE_AJUSTE 0.25
#define COLISION 1
#define DANO_MISIL 3
#define VELOCIDAD_MISIL 0.15
#define VELOCIDAD_BALA 2.5
#define HITPOINTS 10

using namespace std;

enum Estado {INICIO, PASAR_NIVEL, JUGANDO,PAUSA,GAME_OVER, FIN};

void init(bool habilitar_texturas,bool facet_interpol){

    if(facet_interpol)
        glShadeModel(GL_FLAT);
    else
        glShadeModel(GL_SMOOTH);

	glEnable(GL_DEPTH_TEST);
	if(habilitar_texturas)
        glEnable(GL_TEXTURE_2D);
	glClearColor(0,0,0,0);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(ANGULO,ANCHO/ALTO,FRONT,REAR);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void drawSkyBox(GLint textura[6]) {
    float x = 0;
    float y = -1;
    float z = 0;
    float width  = 100;
    float height = 60;
    float length = 100;
    // Bind the BACK texture of the sky map to the BACK side of the cube
    glBindTexture(GL_TEXTURE_2D, textura[0]);
    // Center the skybox
    x = x - width  / 2;
    z = z - length / 2;
    glBegin(GL_QUADS);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,  z);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x, y + height, z);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x, y,  z);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, textura[1]);
    glBegin(GL_QUADS);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y,  z + length);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x, y + height, z + length);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z + length);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,  z + length);
    glEnd();

    glBindTexture(GL_TEXTURE_2D, textura[4]);
    glBegin(GL_QUADS);

        glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y,  z);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x, y,  z + length);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y,  z + length);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,  z);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, textura[5]);
    glBegin(GL_QUADS);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y + height, z);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y + height, z + length);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x, y + height, z + length);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x, y + height, z);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, textura[2]);
    glBegin(GL_QUADS);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x, y + height, z);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x, y + height, z + length);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x, y,  z + length);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x, y,  z);

    glEnd();
    glBindTexture(GL_TEXTURE_2D, textura[3]);
    glBegin(GL_QUADS);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,  z);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,  z + length);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z + length);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z);
    glEnd();
}

/*void drawCube(float x, float y, float z){
	glBegin(GL_QUADS);
        glTexCoord2f(0.0,1.0);
		glVertex3f(  0.5+x, -0.5+y, -0.5+z );
		glTexCoord2f(0.0,0.0);
		glVertex3f(  0.5+x,  0.5+y, -0.5+z );
		glTexCoord2f(1.0,1.0);
		glVertex3f( -0.5+x,  0.5+y, -0.5+z );
		glTexCoord2f(1.0,0.0);
		glVertex3f( -0.5+x, -0.5+y, -0.5+z );
	glEnd();

	// LADO TRASERO: lado blanco
	glBegin(GL_QUADS);
        glTexCoord2f(1.0,0.0);
        glVertex3f(  0.5+x, -0.5+y, 0.5+z );
        glTexCoord2f(0.0,0.0);
        glVertex3f(  0.5+x,  0.5+y, 0.5+z );
        glTexCoord2f(0.0,1.0);
        glVertex3f( -0.5+x,  0.5+y, 0.5+z );
        glTexCoord2f(1.0,1.0);
        glVertex3f( -0.5+x, -0.5+y, 0.5+z );
	glEnd();

	// LADO DERECHO: lado morado
	glBegin(GL_QUADS);
        glTexCoord2f(1.0,0.0);
        glVertex3f( 0.5+x, -0.5+y, -0.5+z );
        glTexCoord2f(0.0,1.0);
        glVertex3f( 0.5+x,  0.5+y,-0.5+z );
        glTexCoord2f(1.0,1.0);
        glVertex3f( 0.5+x,  0.5+y,  0.5+z );
        glTexCoord2f(0.0,0.0);
        glVertex3f( 0.5+x, -0.5+y,  0.5+z );
	glEnd();

	// LADO IZQUIERDO: lado verde
	glBegin(GL_QUADS);
        glTexCoord2f(1.0,0.0);
        glVertex3f( -0.5+x, -0.5+y,  0.5+z );
        glTexCoord2f(0.0,0.0);
        glVertex3f( -0.5+x,  0.5+y,  0.5+z );
        glTexCoord2f(1.0,1.0);
        glVertex3f( -0.5+x,  0.5+y, -0.5+z );
        glTexCoord2f(0.0,1.0);
        glVertex3f( -0.5+x, -0.5+y, -0.5+z );
	glEnd();

	// LADO SUPERIOR: lado azul
	glBegin(GL_QUADS);
		glColor3f(   0.0,  0.0,  1.0 );
		glVertex3f(  0.5+x,  0.5+y,  0.5+z );
		glVertex3f(  0.5+x,  0.5+y, -0.5+z );
		glVertex3f( -0.5+x,  0.5+y, -0.5+z );
		glVertex3f( -0.5+x,  0.5+y,  0.5+z );
	glEnd();

	// LADO INFERIOR: lado rojo
	glBegin(GL_QUADS);
		glColor3f(   1.0,  1.0,  0.0 );
		glVertex3f(  0.5+x, -0.5+y, -0.5+z );
		glVertex3f(  0.5+x, -0.5+y,  0.5+z );
		glVertex3f( -0.5+x, -0.5+y,  0.5+z );
		glVertex3f( -0.5+x, -0.5+y, -0.5+z );
	glEnd();
}*/
void dibujarNumero(GLuint textura,int iz,int desx, int ar, int desy){
    glBindTexture(GL_TEXTURE_2D, textura);
    glPushMatrix();
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
            glLoadIdentity();
            glOrtho(0,ANCHO,0,ALTO,-1,1);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glBegin(GL_QUADS);
                glColor3f(1,1,1);
                glTexCoord2f(0.0,1.0);
                glVertex3f(iz,ar,0.1);
                glTexCoord2f(0.0,0.0);
                glVertex3f(iz,ar-desy,0.1);
                glTexCoord2f(1.0,0.0);
                glVertex3f(iz+desx,ar-desy,0.1);
                glTexCoord2f(1.0,1.0);
                glVertex3f(iz+desx,ar,0.1);
            glEnd();
            glMatrixMode(GL_PROJECTION);
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}

int main(int argc, char *argv[]) {
	Uint32 flags = SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_OPENGL;
	float torres_posiciones[3][3];
	Torre* torres[3];
	int en_torre;
	bool moverCam, der_izq, disparo, wire, fin_nivel, habilitar_texturas, moviendo,game_over;
	int nivel, flecha, cant_balas, cant_misiles, ciudades_vivas, yoquese, puntaje;
	float velocidad;
	Estado estado;
	map<int,Proyectil*> balas;
	map<int,Proyectil*> misiles;
	map<int,Proyectil*> Prontos;
    map<int,map<int,Ciudad*> > ciudades;
	SDL_Event evento;

	moverCam = der_izq = disparo = wire = fin_nivel = moviendo = game_over = false;
	habilitar_texturas = true;

    nivel = velocidad = 1;
    puntaje = yoquese = 0;
    cargarConfiguracion("configuracion.xml",ciudades,torres,torres_posiciones);

    torres[0]->getCamara()->Move(F3dVector(torres_posiciones[0][0] ,torres_posiciones[0][1],torres_posiciones[0][2]));
	torres[1]->getCamara()->Move(F3dVector(torres_posiciones[1][0],torres_posiciones[1][1],torres_posiciones[1][2]));
	torres[2]->getCamara()->Move(F3dVector(torres_posiciones[2][0],torres_posiciones[2][1],torres_posiciones[2][2]));
	torres[1]->getCamara()->RotateY(225);
	torres[2]->getCamara()->RotateY(135);

    bool facet_interpol = true;
    bool texturas = false;
    float xil,yil,zil;
    xil=yil=zil = 100;

    obj_type misil;
	Load3DS (&misil,"missile.3DS");

	obj_type cubo;
	Load3DS(&cubo,"cubo.3DS");

    if(SDL_Init(SDL_INIT_VIDEO)<0) {
        cerr << "No se pudo iniciar SDL: " << SDL_GetError() << endl;
        exit(1);
    }

    if(SDL_SetVideoMode(ALTO, ANCHO, 32, flags)==NULL) {
    	cerr << "No se pudo establecer el modo de video: " << SDL_GetError() << endl;
    	exit(1);
    }

    //TEXTURAS JUEGO//
    GLuint textura_torres = consigueimagen("texturas/canon.png");
    GLuint textura_ciudad = consigueimagen("texturas/canon.png");
    GLuint textura_misil = consigueimagen("texturas/missileTexture.jpg");
    GLuint textura_suelo = consigueimagen("texturas/texturaSuelo.jpg");
    GLuint textura_inicio = consigueimagen("texturas/inicio_comenzar.png");
    GLuint textura_game_over = consigueimagen("texturas/gameover.png");

    //TEXTURA NUMEROS//
    GLuint textura_0 = consigueimagen("texturas/0.png");
    GLuint textura_1 = consigueimagen("texturas/1.png");
    GLuint textura_2 = consigueimagen("texturas/2.png");
    GLuint textura_3 = consigueimagen("texturas/3.png");
    GLuint textura_4 = consigueimagen("texturas/4.png");
    GLuint textura_5 = consigueimagen("texturas/5.png");
    GLuint textura_6 = consigueimagen("texturas/6.png");
    GLuint textura_7 = consigueimagen("texturas/7.png");
    GLuint textura_8 = consigueimagen("texturas/8.png");
    GLuint textura_9 = consigueimagen("texturas/9.png");

    GLuint textura_t;
    GLuint textura_bala = consigueimagen("texturas/bala.png");
    GLuint textura_vida1 = consigueimagen("texturas/torre1.png");
    GLuint textura_vida2 = consigueimagen("texturas/torre2.png");
    GLuint textura_vida3 = consigueimagen("texturas/torre3.png");
    GLuint textura_vida4 = consigueimagen("texturas/torre4.png");
    GLuint textura_torresana = consigueimagen("texturas/vidallena.png");
    GLuint textura_torremedia = consigueimagen("texturas/vidamedia.png");
    GLuint textura_torregrave = consigueimagen("texturas/vidabaja.png");

    //TEXTURAS SKY BOX//
    GLint sky_box_textura[6];
    sky_box_textura[0] = consigueimagen("texturas/cielo.jpg");
    sky_box_textura[1] = consigueimagen( "texturas/cielo.jpg");
    sky_box_textura[2] = consigueimagen( "texturas/cielo.jpg");
    sky_box_textura[3] = consigueimagen( "texturas/cielo.jpg");
    sky_box_textura[4] = consigueimagen( "texturas/texturaSuelo.jpg");
    sky_box_textura[5] = consigueimagen( "texturas/cielo.jpg");
    GLuint textura3;

    //TEXXTURAS MENU
    GLuint MENU_Selecc_Continuar = consigueimagen("texturas/MENU_Selecc_Continuar.jpg");
    GLuint MENU_Selecc_Velocidad = consigueimagen("texturas/MENU_Selecc_Velocidad.jpg");
    GLuint MENU_Selecc_Estados = consigueimagen("texturas/MENU_Selecc_Estados.jpg");
    GLuint MENU_Selecc_DirecyColor = consigueimagen("texturas/MENU_Selecc_DirecyColor.jpg");
    GLuint MENU_VELOCIDAD_Lento = consigueimagen("texturas/MENU_VELOCIDAD_Lento.png");
    GLuint MENU_VELOCIDAD_Normal = consigueimagen("texturas/MENU_VELOCIDAD_Normal.png");
    GLuint MENU_VELOCIDAD_Rapido = consigueimagen("texturas/MENU_VELOCIDAD_Rapido.png");
    GLuint MENU_ESTADOS_Selecc_Wireframe = consigueimagen("texturas/MENU_ESTADOS_Selecc_Wireframe.png");
    GLuint MENU_ESTADOS_Selecc_Facet_Inter = consigueimagen("texturas/MENU_ESTADOS_Selecc_Facet_Inter.png");
    GLuint MENU_ESTADOS_Selecc_Texturas = consigueimagen("texturas/MENU_ESTADOS_Selecc_Texturas.png");
    GLuint MENU_ROJO = consigueimagen("texturas/MENU_ROJO.png");
    GLuint MENU_AZUL = consigueimagen("texturas/MENU_AZUL.png");
    GLuint MENU_VERDE = consigueimagen("texturas/MENU_VERDE.png");
    GLuint MENU_BLANCO = consigueimagen("texturas/MENU_BLANCO.png");
    GLuint MENU_DIREC = consigueimagen("texturas/MENU_DIREC.png");
    GLuint MENU_inicio_comenzar = consigueimagen("texturas/inicio_comenzar.png");
    GLuint MENU_inicio_salir = consigueimagen("texturas/inicio_salir.png");
    GLuint menu_torre1 = consigueimagen("texturas/torre1d.jpg");
    GLuint menu_torre2 = consigueimagen("texturas/torre2d.jpg");
    GLuint menu_torre3 = consigueimagen("texturas/torre3d.jpg");

    misil.id_texture = textura_misil;
    cubo.id_texture = textura_ciudad;
    float x,z; //Para el cambio de torre

    init(habilitar_texturas,facet_interpol);
    estado = INICIO;
    float red,green,blue;
    red=green=blue=255;
    bool lumus = false;

    do{
        fin_nivel = moverCam = false;
        flecha = en_torre = cant_misiles = cant_balas = 0;
        ciudades_vivas = nivel+1;

        for(map<int,Proyectil*>::iterator it=balas.begin();it!=balas.end();++it)
            delete it->second;
        balas.clear();

        for(map<int,Proyectil*>::iterator it=misiles.begin();it!=misiles.end();++it)
            delete it->second;
        misiles.clear();

        for(int i=0;i<3;i++)
            torres[i]->reestablecerMunicion();

        int tirados=0;
        int vueltas=0;

        do{

            int start_time = SDL_GetTicks();

            //Si cayeron todos los misiles
            if((cant_misiles == 10*nivel && tirados==0))
                estado = PASAR_NIVEL;

            //Si supere la cantidad maxima de niveles del juego
            if(nivel > 3)
                    estado = GAME_OVER;

            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

            if(estado == JUGANDO){
                vueltas++;
                //RANDOM PARA TIRAR LOS MISILES Y SI DA 300 LOOPS TIRA UNO (POR SI AL RANDOM NO SE LE DA LA GANA EMBOCARLE AL MODULO)
                if (tirados < 5 && cant_misiles < (10*nivel)){
                    if ((((rand())%(200/nivel))==5)||(vueltas>300)){
                        int que_ciudad;
                        do{
                            que_ciudad = rand()%(nivel+1);
                        }while(!ciudades[nivel][que_ciudad]->getActivo() && estado != GAME_OVER);// !game_over);

                        float x = pow(-1,rand()%2)*(rand()%25 + 10);
                        float y = 100;
                        float z = pow(-1,rand()%2)*(rand()%25 + 10);
                        float viewX = ciudades[nivel][que_ciudad]->getX()-x;
                        float viewY = ciudades[nivel][que_ciudad]->getY()-y;
                        float viewZ = ciudades[nivel][que_ciudad]->getZ()-z;
                        Proyectil* m = new Proyectil(&misil,x,y,z,viewX,viewY,viewZ);
                        misiles[cant_misiles] = m;
                        cant_misiles++;
                        vueltas=0;
                        tirados++;
                    }
                }

                //Renderizo la camara
                if(moviendo){
                    glLoadIdentity();
                    if(torres_posiciones[en_torre][0] != x || torres_posiciones[en_torre][2] != z){
                        gluLookAt(x,0,z,0,0,0,0,1,0);
                        if(torres_posiciones[en_torre][0] > x)
                            x += 0.25*velocidad;
                        else
                            if(torres_posiciones[en_torre][0] < x)
                                x -= 0.25*velocidad;
                        if(torres_posiciones[en_torre][2] > z){
                            z += 0.5*velocidad;
                        }
                        else
                        if(torres_posiciones[en_torre][2] < z){
                                z -= 0.5*velocidad;
                        }
                    }
                    else
                        moviendo = false;
                }
                else{
                    glLoadIdentity();
                    torres[en_torre]->getCamara()->Render();
                }

                //PISO//
                glPushMatrix();
                    glColor3f(0,0,0);
                    drawSkyBox(sky_box_textura);
                glPopMatrix();

                //CIUDADES//
                glBindTexture(GL_TEXTURE_2D, textura_ciudad);
                for(map<int,Ciudad*>::iterator c=ciudades[nivel].begin();c!=ciudades[nivel].end();++c){
                    glPushMatrix();
                        glColor3f(1,0,0);
                        glTranslated(c->second->getX(),c->second->getY(),c->second->getZ());
                        drawObject(cubo);
                    glPopMatrix();
                }

                //RENDERIZAR LAS TORRES//
                for(int i=0;i<3;i++){
                    if(!(en_torre == i)){
                        glPushMatrix();
                        glColor3f(0,1,0);
                            glTranslatef(torres_posiciones[i][0],torres_posiciones[i][1],torres_posiciones[i][2]);
                            glScalef(0.4,0.4,0.4);
                            drawObject(cubo);
                        glPopMatrix();
                    }
                }

                //RENDERIZAR LAS BALAS//
                for(map<int,Proyectil*>::iterator b=balas.begin();b!=balas.end();++b){
                    if(b->second->getY() >= 0 && b->second->getActivo() && !moviendo){
                        b->second->recalcularPosicion(VELOCIDAD_BALA);
                        float bx = b->second->getX();
                        float by = b->second->getY();
                        float bz = b->second->getZ();

                        //CONTROL DE COLISION CON MISIL//
                        for(map<int,Proyectil*>::iterator m=misiles.begin();m!=misiles.end();++m){
                            if(m->second->getActivo()){
                                float mx = m->second->getX();
                                float my = m->second->getY();
                                float mz = m->second->getZ();
                                float distancia = sqrt(pow(bx-mx,2)+pow(by-my,2)+pow(bz-mz,2));
                                if(distancia < COLISION){
                                    puntaje += HITPOINTS;
                                    b->second->desactivar();
                                    m->second->desactivar();
                                    tirados--;
                                    break;
                                }
                            }
                        }

                        glPushMatrix();
                            glColor3f(0,0,1);
                            glTranslatef(bx,by,bz);
                            glScalef(0.00025,0.00025,0.00025);
                            drawObject(*b->second->getModelo());
                        glPopMatrix();
                    }
                }

                //RENDERIZAR LOS MISILES//
                for(map<int,Proyectil*>::iterator m=misiles.begin();m!=misiles.end();++m){
                    if(m->second->getY() >= 0 && m->second->getActivo() && !moviendo){
                        m->second->recalcularPosicion(velocidad*VELOCIDAD_MISIL+nivel/10);

                        float mx = m->second->getX();
                        float my = m->second->getY();
                        float mz = m->second->getZ();
                        //CONTROL DE COLISION CON ciudades//
                        for(map<int,Ciudad*>::iterator c=ciudades[nivel].begin();c!=ciudades[nivel].end();++c){
                            float cx = c->second->getX();
                            float cy = c->second->getY();
                            float cz = c->second->getZ();
                            float distancia = sqrt(pow(cx-mx,2)+pow(cy-my,2)+pow(cz-mz,2));
                            if(distancia < COLISION){
                                c->second->decrementarSalud(DANO_MISIL);
                                m->second->desactivar();
                                tirados--;
                                if(c->second->getSalud() <= 0){
                                        c->second->desactivar();
                                        ciudades_vivas--;
                                        if(ciudades_vivas == 0)
                                            estado = GAME_OVER;
                                }
                                break;
                            }
                        }

                        glPushMatrix();
                            glColor3f(0,0,1);
                            glTranslatef(m->second->getX(),m->second->getY(),m->second->getZ());
                            glRotatef(180,1,0,0);
                            glScalef(0.001,0.001,0.001);
                            drawObject(misil);
                        glPopMatrix();
                    }
                    else if(m->second->getY() < 0){
                        m->second->desactivar();
                        tirados--;
                    }
                }

                //RENDERIZO EL HUD//
                GLuint textura_aux, textura_1000,textura_100,textura_10,textura_uno;
                int arr[4]={0};
                //unidades
                arr[0]= puntaje%10;
                //decenas
                arr[1]=((puntaje-arr[0])/10)%10;
                //centenas
                arr[2]=((puntaje-arr[1]-arr[0])/100)%10;
                //mil
                arr[3]=((puntaje-arr[2]-arr[1]-arr[0])/1000)%10;

                for(int i=0;i<4;i++)
                    {
                        switch (arr[i]){
                        case 0:
                            textura_aux=textura_0;
                            break;
                        case 1:
                            textura_aux=textura_1;
                            break;
                        case 2:
                            textura_aux=textura_2;
                            break;
                        case 3:
                            textura_aux=textura_3;
                            break;
                        case 4:
                            textura_aux=textura_4;
                            break;
                        case 5:
                            textura_aux=textura_5;
                            break;
                        case 6:
                            textura_aux=textura_6;
                            break;
                        case 7:
                            textura_aux=textura_7;
                            break;
                        case 8:
                            textura_aux=textura_8;
                            break;
                        case 9:
                            textura_aux=textura_9;
                            break;
                        }

                        switch (i)
                        {
                            case 0:
                                textura_uno=textura_aux;
                                break;
                            case 1:
                                textura_10=textura_aux;
                                break;
                            case 2:
                                textura_100=textura_aux;
                                break;
                            case 3:
                                textura_1000=textura_aux;
                                break;
                        }
                }

                glDisable(GL_LIGHTING);
                glDisable(GL_LIGHT0);
                //mil
                dibujarNumero(textura_1000,0,35,ALTO,30);
                //cien
                dibujarNumero(textura_100,30,35,ALTO,30);
                //diez
                dibujarNumero(textura_10,60,35,ALTO,30);
                //uno
                dibujarNumero(textura_uno,90,35,ALTO,30);

                for(int i=0; i<torres[en_torre]->getMunicion();i++)
                    dibujarNumero(textura_bala,200+(i*5),5,100,60);

                // RENDERIZO MIRA
                dibujarNumero(textura_uno,238,4,318,6);

                //RENDERIZO LA VIDA DE LAS TORRES
                for(map<int,Ciudad*>::iterator it=ciudades[nivel].begin();it!=ciudades[nivel].end();++it){
                    int y;
                    switch (it->first){
                        case 0:
                            textura_t=textura_vida1;
                            y=50;
                            break;
                        case 1:
                            textura_t=textura_vida2;
                            y=50+20;
                            break;
                        case 2:
                            textura_t=textura_vida3;
                            y=50+40;
                            break;
                        case 3:
                            textura_t=textura_vida4;
                            y=50+60;
                            break;
                    }
                    int pos= 93;
                    dibujarNumero(textura_t,10,83,ALTO-y,20);
                    int sal= it->second->getSalud();
                    for(int i=0; i<sal;i++){
                        if (sal< (it->second->getSaludTot()/3))
                            dibujarNumero(textura_torregrave,pos,13,ALTO-y,20);
                        else
                            if (sal< (it->second->getSaludTot()/2))
                                dibujarNumero(textura_torremedia,pos,13,ALTO-y,20);
                            else
                                dibujarNumero(textura_torresana,pos,13,ALTO-y,20);
                        pos+=13;
                    }
                }

                //SI LA ILUMINACION ESTA HABLILITADA
                if(lumus)
                    iluminar(red,green,blue,xil,yil,zil);
            }

            else{
                if(estado == PAUSA){ //Estoy en pausa, despliego los menus correspondientes segun donde este ubicado
                    switch(yoquese){
                        case 0:
                            switch (flecha){// MENU PRINCIPAL
                                case -1:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_Continuar);
                                    flecha=0;
                                    break;

                                case 0:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_Continuar);
                                    break;

                                case 1:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_Velocidad);
                                    break;

                                case 2:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_Estados);
                                    break;

                                case 3:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_DirecyColor);
                                    break;

                                case 4:
                                    glBindTexture(GL_TEXTURE_2D, MENU_Selecc_DirecyColor);
                                    flecha =3;
                                    break;
                            }
                            break;

                        case 1:
                            switch(flecha){ //SUBMENU DE VELOCIDAD
                                case 0:
                                    flecha=1;
                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Lento);
                                    break;

                                case 1:
                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Lento);
                                    break;

                                case 2:
                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Normal);
                                    break;

                                case 3:
                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Rapido);
                                    break;

                                case 4:
                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Rapido);
                                    flecha =3;
                                    break;
                            }
                            break;

                        case 2:
                            switch(flecha){ //SUBMENU WIREFRAME/TEXTURAS/SOMBRAS
                                case 1:
                                    flecha=2;
                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Wireframe);
                                    break;
                                case 2:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Wireframe);
                                    break;
                                case 3:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Facet_Inter);
                                    break;
                                case 4:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Texturas);
                                    break;
                                case 5:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Texturas);
                                    flecha =4;
                                    break;
                                }
                            break;

                        case 3:
                            switch(flecha){ //SUBMENU COLORES DE LUZ
                                case 2:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ROJO);
                                    flecha =3;
                                    break;
                                case 3:
                                    glBindTexture(GL_TEXTURE_2D, MENU_ROJO);
                                    break;
                                case 4:
                                    glBindTexture(GL_TEXTURE_2D, MENU_AZUL);
                                    break;
                                case 5:
                                    glBindTexture(GL_TEXTURE_2D, MENU_VERDE);
                                    break;
                                case 6:
                                    glBindTexture(GL_TEXTURE_2D, MENU_BLANCO);
                                    break;
                                case 7:
                                    glBindTexture(GL_TEXTURE_2D, MENU_DIREC);
                                    break;
                                case 8:
                                    glBindTexture(GL_TEXTURE_2D, MENU_DIREC);
                                    flecha =7;
                                    break;
                            }
                            break;
                        case 4:
                               switch(flecha){ //SUBMENU DE DIRECCION DE LA LUZ
                                case 6:
                                    glBindTexture(GL_TEXTURE_2D,menu_torre1);
                                    flecha = 7;
                                    break;
                                case 7:
                                    glBindTexture(GL_TEXTURE_2D,menu_torre1);
                                    break;
                                case 8:
                                    glBindTexture(GL_TEXTURE_2D,menu_torre2);
                                    break;
                                case 9:
                                    glBindTexture(GL_TEXTURE_2D,menu_torre3);
                                    break;
                                case 10:
                                    glBindTexture(GL_TEXTURE_2D,menu_torre3);
                                    flecha = 9;
                                    break;
                               }
                    }
                }

                else if(estado == INICIO){ //MENU DEL COM]IENZO DEL JUEGO
                    switch(flecha){
                    case -1:
                        flecha=0;
                        glBindTexture(GL_TEXTURE_2D, MENU_inicio_comenzar);
                        break;
                    case 0:
                        glBindTexture(GL_TEXTURE_2D, MENU_inicio_comenzar);
                        break;
                    case 1:
                        glBindTexture(GL_TEXTURE_2D, MENU_inicio_salir);
                        break;
                    case 2:
                        flecha = 1;
                        glBindTexture(GL_TEXTURE_2D, MENU_inicio_salir);
                        break;
                    }
                }

                if(estado == GAME_OVER)
                    glBindTexture(GL_TEXTURE_2D,textura_game_over);

                glEnable(GL_TEXTURE_2D);
                glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
                glDisable(GL_LIGHTING);
                glDisable(GL_LIGHT0);
                glPushMatrix();
                    glMatrixMode(GL_PROJECTION);
                    glPushMatrix();
                    glLoadIdentity();
                    glOrtho(-10,10,-10,10,-1,1);
                    glPushMatrix();
                    glMatrixMode(GL_MODELVIEW);
                    glLoadIdentity();
                    glBegin(GL_QUADS);
                        glTexCoord2f(0.0,0.0);
                        glVertex3f(-10,-10,0);
                        glTexCoord2f(0.0,1.0);
                        glVertex3f(-10,10,0);
                        glTexCoord2f(1.0,1.0);
                        glVertex3f(10,10,0);
                        glTexCoord2f(1.0,0.0);
                        glVertex3f(10,-10,0);
                    glEnd();
                glMatrixMode(GL_PROJECTION);
                glPopMatrix();
                glMatrixMode(GL_MODELVIEW);
                glPopMatrix();
                glDisable(GL_TEXTURE_2D);

                if(wire)
                    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
                else
                    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

                if(lumus)
                    iluminar(red,green,blue,xil,yil,zil);

            }


            while(SDL_PollEvent(&evento)){

                switch(evento.type){

                    case SDL_MOUSEBUTTONDOWN:
                        if(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3) && estado == JUGANDO){
                            moverCam=true;
                        }
                        else if(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(1) && estado == JUGANDO){
                            if(torres[en_torre]->getMunicion() > 0){
                                torres[en_torre]->disparar();
                                float x = torres[en_torre]->getCamara()->getPosition().x;
                                float y = torres[en_torre]->getCamara()->getPosition().y;
                                float z = torres[en_torre]->getCamara()->getPosition().z;
                                float viewX = torres[en_torre]->getCamara()->getViewDir().x;
                                float viewY = torres[en_torre]->getCamara()->getViewDir().y;
                                float viewZ = torres[en_torre]->getCamara()->getViewDir().z;
                                Proyectil*  b = new Proyectil(&misil,x,y,z,viewX,viewY,viewZ);
                                balas[cant_balas] = b;
                                cant_balas++;
                            }
                        }
                        break;
                    case SDL_MOUSEBUTTONUP:
                        if(SDL_BUTTON_RIGHT && moverCam){
                            moverCam=false;
                        }
                        break;
                    case SDL_MOUSEMOTION:
                        if(moverCam){
                            torres[en_torre]->getCamara()->RotateX(-evento.motion.yrel*FACTOR_DE_AJUSTE);
                            torres[en_torre]->getCamara()->RotateY(-evento.motion.xrel*FACTOR_DE_AJUSTE);
                        }
                        break;
                    case SDL_QUIT:
                    case SDLK_q:
                        fin_nivel = true;
                        break;
                    case SDL_KEYDOWN:
                        switch(evento.key.keysym.sym){
                            default:
                                break;

                            case SDLK_t:
                                habilitar_texturas = !habilitar_texturas;
                                if(habilitar_texturas)
                                    glEnable(GL_TEXTURE_2D);
                                else
                                    glDisable(GL_TEXTURE_2D);
                                break;

                            case SDLK_m:
                                nivel++;
                                break;

                            case SDLK_DOWN:
                                if(estado == PAUSA || estado == INICIO)
                                    flecha++;
                                break;

                            case SDLK_UP:
                                if(estado == PAUSA || estado == INICIO)
                                    flecha--;
                                break;

                            case SDLK_p:
                                if(estado == PAUSA && estado != INICIO)
                                    estado = JUGANDO;
                                else if(estado == JUGANDO)
                                    estado = PAUSA;

                                yoquese=0;
                                init(habilitar_texturas,facet_interpol);
                                flecha=0;
                                break;

                            case SDLK_KP_ENTER:
                            case SDLK_RETURN:
                                    if(estado == PAUSA){
                                        if(yoquese==0){
                                            switch(flecha){
                                                case 0:
                                                    //CONTINUAR JUGANDO
                                                    estado = JUGANDO;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 1:
                                                    // CAMBIO AL MENU DE VELOCIDAD
                                                    glBindTexture(GL_TEXTURE_2D, MENU_VELOCIDAD_Lento);
                                                    yoquese=1;
                                                    break;

                                                case 2:
                                                    //CAMBIO AL MENU DE ESTADOS
                                                    glBindTexture(GL_TEXTURE_2D, MENU_ESTADOS_Selecc_Wireframe);
                                                    yoquese=2;
                                                    break;

                                                case 3:
                                                    //CAMBIO AL MENU DE DIRECYVCOLOR
                                                    yoquese=3;
                                                    break;

                                                default:
                                                    break;
                                            }
                                        }
                                        else if(yoquese==1){
                                            estado = JUGANDO;
                                            switch(flecha){
                                               case 1:
                                                    velocidad=0.5;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                               case 2:
                                                    velocidad=1;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                               case 3:
                                                    velocidad=2;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;
                                            }
                                        }
                                        else if(yoquese==2){
                                            estado = JUGANDO;
                                            switch(flecha){
                                                case 2:
                                                    // WIREFRAME
                                                    if(!wire)
                                                        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
                                                    else
                                                        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
                                                    wire = !wire;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                    case 3:
                                                    // FACETADO/INTERPOLADO
                                                    facet_interpol = !facet_interpol;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 4:
                                                    //TEXTURA
                                                    habilitar_texturas = !habilitar_texturas;
                                                    if(habilitar_texturas)
                                                        glEnable(GL_TEXTURE_2D);
                                                    else
                                                        glDisable(GL_TEXTURE_2D);
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;
                                            }
                                        }
                                        else if(yoquese==3){
                                           // estado = JUGANDO;
                                            switch(flecha){
                                                case 3:
                                                    // ROJO
                                                    estado = JUGANDO;
                                                    lumus=true;
                                                    red = 255;
                                                    green=blue=0;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 4:
                                                    //AZUL
                                                    estado = JUGANDO;
                                                    lumus=true;
                                                    blue=255;
                                                    red=green=0;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 5:
                                                    // VERDE
                                                    estado = JUGANDO;
                                                    lumus=true;
                                                    green=255;
                                                    red=blue=0;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 6:
                                                    //BLANCA
                                                    estado = JUGANDO;
                                                    lumus=true;
                                                    green=red=blue=255;
                                                    init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;

                                                case 7:
                                                    //DIRECCION
                                                    glBindTexture(GL_TEXTURE_2D,menu_torre1);
                                                    yoquese=4;
                                                    break;
                                            }
                                        }else if(yoquese==4){
                                            estado = JUGANDO;
                                            switch(flecha){
                                            case 7:
                                                xil = torres_posiciones[0][0];
                                                yil = torres_posiciones[0][1];
                                                zil = torres_posiciones[0][2];
                                                init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                break;
                                            case 8:
                                                xil = torres_posiciones[1][0];
                                                yil = torres_posiciones[1][1];
                                                zil = torres_posiciones[1][2];
                                                init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;
                                            case 9:
                                                xil = torres_posiciones[2][0];
                                                yil = torres_posiciones[2][1];
                                                zil = torres_posiciones[2][2];
                                                init(habilitar_texturas,facet_interpol);
                                                    flecha=0;
                                                    yoquese=0;
                                                    break;
                                            }
                                        }
                                    }
                                    else if(estado == INICIO || estado == PASAR_NIVEL){
                                        switch(flecha){
                                            case 0:
                                                estado = JUGANDO;
                                                init(habilitar_texturas,facet_interpol);
                                                flecha=0;
                                                yoquese=0;
                                                break;

                                            case 1:
                                                estado = GAME_OVER;
                                                break;
                                        }
                                    }
                                    else if(estado == GAME_OVER)
                                        estado = FIN;
                                break;

                            case SDLK_SPACE:
                                if(!wire)
                                    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
                                else
                                    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
                                wire = !wire;
                                break;
                            case SDLK_ESCAPE:
                            case SDLK_q:
                                estado = GAME_OVER;
                                break;

                            case SDLK_1:
                                if(!moviendo && en_torre != 0){
                                    x = torres_posiciones[en_torre][0];
                                    z = torres_posiciones[en_torre][2];
                                    en_torre = 0;
                                    moviendo = true;
                                }
                                break;

                            case SDLK_2:
                                if(!moviendo && en_torre != 1){
                                    x = torres_posiciones[en_torre][0];
                                    z = torres_posiciones[en_torre][2];
                                    en_torre = 1;
                                    moviendo = true;
                                }
                                break;

                            case SDLK_3:
                                if(!moviendo && en_torre != 2){
                                    x = torres_posiciones[en_torre][0];
                                    z = torres_posiciones[en_torre][2];
                                    en_torre = 2;
                                    moviendo = true;
                                }
                                break;

                            case SDLK_F11:
                                flags ^= SDL_FULLSCREEN;
                                if(SDL_SetVideoMode(640, 480, 32, flags)==NULL) {
                                    fprintf(stderr, "No se pudo establecer el modo de video: %s\n", SDL_GetError());
                                    exit(1);
                                }
                                glMatrixMode(GL_PROJECTION);
                                float color = 0;
                                glClearColor(color, color, color, 1);
                                gluPerspective(ANGULO, ALTO/ANCHO, FRONT, REAR);
                                glMatrixMode(GL_MODELVIEW);
                                break;
                            }
                            break;

                    default:
                        break;
                    }
            }
            SDL_GL_SwapBuffers();

            if((1000/FPS)>(SDL_GetTicks()-start_time)){
                SDL_Delay((1000/FPS)-(SDL_GetTicks()-start_time));
            }


        }while(estado != PASAR_NIVEL && estado != GAME_OVER && estado != FIN);

        if(estado != GAME_OVER && estado != FIN){
            nivel++;
            estado = PAUSA;
        }

    }while(estado != FIN);

    for(int i=0;i<3;i++)
        delete torres[i];
    balas.clear();
    misiles.clear();
    SDL_Quit();
    return 0;
}
