#include "..\headers\Asteroids.h"

Asteroid::Asteroid()
{
	SDL_Surface* img = SDL_LoadBMP(img_textura_asteroids);
	glGenTextures(1, &textura);
	glBindTexture(GL_TEXTURE_2D, textura);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img->w, img->h, 0, GL_BGR, GL_UNSIGNED_BYTE, img->pixels);
	SDL_FreeSurface(img);

	for (int i = 0; i < numSteps; i++)
	{
		double lat0 = M_PI * (-0.5 + (double)(i - 1) / numSteps);
		double z0 = sin(lat0);
		double zr0 = cos(lat0);

		double lat1 = M_PI * (-0.5 + (double)i / numSteps);
		double z1 = sin(lat1);
		double zr1 = cos(lat1);

		for (int j = 0; j < numSteps; j++)
		{
			double lng = 2 * M_PI * (double)(j - 1) / numSteps;
			double x = cos(lng);
			double y = sin(lng);

			double s1, s2, t;
			s1 = ((double)i) / numSteps;
			s2 = ((double)i + 1) / numSteps;
			t = ((double)j) / numSteps;

			matriz[i][j][1] = radioNormal*x * zr0 * (rand() % 100 / 75);
			matriz[i][j][2] = -radioNormal*z0;
			matriz[i][j][3] = radioNormal*y * zr0 * (rand() % 100 / 75);
			matriz[i][j][4] = radioNormal*x * zr1 * (rand() % 100 / 75);
			matriz[i][j][5] = -radioNormal*z1;
			matriz[i][j][6] = radioNormal* y * zr1 * (rand() % 100 / 75);
		}
	}

	isNull = true;
}

void Asteroid::setValores(float X, float Y, float Z, Vector3D vCaida)
{
	isNull = false;
	radius = 2;
	numPts = 8;
	posActual = Vector3D(X, Y, Z);
	vCaida.normalizar();
	velocidad = vCaida * velocidadAsteriode;
}

void drawAsteroid(int lats, int longs, GLfloat r, float matriz[numSteps][numSteps][6])
{
	int i, j;
	double s1, s2, t;

	for (i = 0; i < lats; i++)
	{
		glBegin(GL_QUAD_STRIP);
		for (j = 0; j < longs; j++)
		{
			s1 = ((double)i) / lats;
			s2 = ((double)i + 1) / lats;
			t = ((double)j) / longs;

			glTexCoord2d(s1, t);
			glNormal3f(matriz[i][j][1], matriz[i][j][2] / -r, matriz[i][j][3]);
			glVertex3f(matriz[i][j][1], matriz[i][j][2], matriz[i][j][3]);

			glTexCoord2d(s2, t);
			glNormal3f(matriz[i][j][4], matriz[i][j][5] / -r, matriz[i][j][6]);
			glVertex3f(matriz[i][j][4], matriz[i][j][5], matriz[i][j][6]);
		}
		glEnd();
	}
}

void Asteroid::draw(bool mov)
{
	glPushMatrix();
	
	glBindTexture(GL_TEXTURE_2D, textura);
	
	if (mov)
	{
		posActual += velocidad * GeneralSettings::Instance()->getVelocidad();
	}

	glTranslatef(posActual.getX(), posActual.getY(), posActual.getZ());
	drawAsteroid(numSteps, numSteps, radioNormal, matriz);
	
	glPopMatrix();
}

float Asteroid::getX()
{
	return posActual.getX();
}

float Asteroid::getY()
{
	return posActual.getY() + radioNormal;
}

float Asteroid::getZ()
{
	return posActual.getZ();
}

bool Asteroid::IsNull()
{
	return isNull;
}

void Asteroid::setNull()
{
	isNull = true;
}

Asteroid::~Asteroid()
{
}
