#include "..\headers\Bullet.h"

Bullet::Bullet()
{
	//glGenTextures(1, &textura);
	SDL_Surface* img = SDL_LoadBMP(img_textura_bullet);
//	glGenTextures(1, &textura);
	glBindTexture(GL_TEXTURE_2D, textura);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img->w, img->h, 0, GL_BGR, GL_UNSIGNED_BYTE, img->pixels);
	SDL_FreeSurface(img);

	isNull = true;
	nextEstela = 0;
}

void Bullet::setValores(float X, float Y, float Z, float angX, float angY)
{
	isNull = false;
	
	posActual = Vector3D(X, Y, Z);

	velocidad = Vector3D(0.0, 1.0, 0.0);
	Vector3D rotarX = Vector3D(posActual.getX(), posActual.getY(), posActual.getZ());
	rotarX.normalizar();
	velocidad = velocidad.rotar(rotarX, angX);
	
	Vector3D enY;
	if (X != 0)
	{
		if (X < 0)
			enY = Vector3D(Z / X, 0.0, -1.0);
		else
			enY = Vector3D(-Z / X, 0.0, 1.0);
	}
	else
		enY = Vector3D(-Z, 0.0, 0.0);
	enY.normalizar();
	velocidad = velocidad.rotar(enY, 90 - angY);
	velocidad.normalizar();
	velocidad = velocidad * 0.5;
}

void drawSphere(int lats, int longs, GLfloat r)
{

	int i, j;
	for (i = 0; i <= lats; i++)
	{
		double lat0 = M_PI * (-0.5 + (double)(i - 1) / lats);
		double z0 = sin(lat0);
		double zr0 = cos(lat0);

		double lat1 = M_PI * (-0.5 + (double)i / lats);
		double z1 = sin(lat1);
		double zr1 = cos(lat1);

		glBegin(GL_QUAD_STRIP);
		for (j = 0; j <= longs; j++)
		{
			double lng = 2 * M_PI * (double)(j - 1) / longs;
			double x = cos(lng);
			double y = sin(lng);

			double s1, s2, t;
			s1 = ((double)i) / lats;
			s2 = ((double)i + 1) / lats;
			t = ((double)j) / longs;

			glTexCoord2d(s1, t);
			glNormal3d(x * zr0, z0, y * zr0);
			glVertex3d(r*x * zr0, -r*z0, r*y * zr0);

			glTexCoord2d(s2, t);
			glNormal3d(x * zr1, z1, y * zr1);
			glVertex3d(r*x * zr1, -r*z1, r* y * zr1);
		}
		glEnd();
	}

}

void Bullet::draw(bool mov)
{
	glBindTexture(GL_TEXTURE_2D, textura);
	glPushMatrix();
	
	if (mov)
	{
		posActual += velocidad * GeneralSettings::Instance()->getVelocidad();
	}

	glTranslatef(posActual.getX(), posActual.getY()-1.0, posActual.getZ());
	
	drawSphere(num_step, num_step, radio);
	
	glPopMatrix();
	
	glPushMatrix();
	
	Vector3D vectorEstela = posActual;
	vectorEstela.setXYZ(vectorEstela.getX(), vectorEstela.getY() - 1.0, vectorEstela.getZ());

	if (mov)
	{
		explo[nextEstela].setEstela(vectorEstela);
		nextEstela = (nextEstela + 1) % cantEstela;
	}
	
	for (int i = 0; i < cantEstela; i++){
		explo[i].drawEst(velocidad, mov);
	}
	glPopMatrix();
}

float Bullet::getX()
{
	return posActual.getX();
}

float Bullet::getY()
{
	return posActual.getY();
}

float Bullet::getZ()
{
	return posActual.getZ();
}

bool Bullet::IsNull()
{
	return isNull;
}

void Bullet::setNull()
{
	isNull = true;
}

void Bullet::destEst()
{
	for (int i = 0; i < cantEstela; i++)
	{
		explo[i].setNull();
	}
}

Bullet::~Bullet()
{
}
