#include "..\headers\Colisionador.h"

Colisionador::Colisionador(int cantBull, int cantAst, int cantSat)
{
	cantBullets = cantBull;
	cantAsteroids = cantAst;
	cantSatelites = cantSat;

	cantAstDestruidos = 0;

	bull = new Bullet[cantBullets];
	explo = new Explosion[cantAsteroids];
	ast = new Asteroid[cantAsteroids];
	sat = new Modelo[cantSatelites];
	posXSatelite = new float[cantSatelites];
	posZSatelite = new float[cantSatelites];

	cantSatDestruidos = 0;
	puntajeAnt = 0;

	if (cantSatelites == 1)
	{
		sat[0].setValores(0.0, posYSatelite, 0.0);
		posXSatelite[0] = 0;
		posZSatelite[0] = 0;
	}
	else
	{
		float angSat = 360.0 / cantSatelites;
		float angAux = 0;
		for (int i = 0; i < cantSatelites; i++)
		{
			posXSatelite[i] = 5.0 * sin(angAux * M_PI / 180);
			posZSatelite[i] = 5.0 * cos(angAux * M_PI / 180);

			sat[i].setValores(posXSatelite[i], posYSatelite, posZSatelite[i]);

			angAux += angSat;
		}
	}

	tAsteroide = SDL_GetTicks();
}

void Colisionador::newBullet(float posX, float posY, float posZ, float angX, float angY)
{
	for (int i = 0; i < cantBullets; i++){
		if (bull[i].IsNull())
		{
			bull[i].setValores(posX, posY, posZ, angX, angY);
			break;
		}
	}
}

void Colisionador::newAsteroid()
{
	for (int i = 0; i < cantAsteroids; i++){
		if (ast[i].IsNull())
		{
			float ang = rand() % 360;
			int cualSat = rand() % cantSatelites;
			if (sat[cualSat].IsNull() && (cantSatDestruidos < cantSatelites))
			{
				while (sat[cualSat].IsNull())
				{
					cualSat = (cualSat + 1) % cantSatelites;
				}
			}
			float alturaAst = 15;
			Vector3D recorrido = Vector3D(posXSatelite[cualSat] - 20 * sin(ang * M_PI / 180),
										posYSatelite - alturaAst,
										posZSatelite[cualSat] - 20 * sin(ang * M_PI / 180));
			ast[i].setValores(20 * sin(ang * M_PI / 180), alturaAst, 20 * sin(ang * M_PI / 180), recorrido);
			break;
		}
	}
}

void Colisionador::colisionar()
{
	float distCentros;
	bool destruirBala = false;

	// Choque de la bala
	for (int bullId = 0; bullId < cantBullets; bullId++)
	{
		destruirBala = false;
		if (!bull[bullId].IsNull()){
			for (int astId = 0; astId < cantAsteroids; astId++)
			{
				if (!ast[astId].IsNull())
				{
					// Choque contra asteroide
					distCentros = (abs(bull[bullId].getX() - ast[astId].getX())) +
						abs(bull[bullId].getY() - ast[astId].getY()) +
						abs(bull[bullId].getZ() - ast[astId].getZ());
					if (distCentros < 1.9)
					{
						// Si choque, destruccin asteroide
						ast[astId].~Asteroid();
						ast[astId].setNull();

						cantAstDestruidos++;

						for (int e = 0; e < cantAsteroids; e++){
							if (explo[e].IsNull())
							{
								explo[e].setPosicionExp(Vector3D(ast[astId].getX(),
									ast[astId].getY()-1.5, ast[astId].getZ()));
								break;
							}
						}

						destruirBala = true;
						break;
					}
				}
			}

			// Destruccin de la bala por choque o fuera del rea de importancia
			if ((destruirBala) || ((abs(bull[bullId].getX())) > 30) || ((abs(bull[bullId].getY())) > 20) || ((abs(bull[bullId].getZ())) > 30.0))
			{
				bull[bullId].setNull();
				bull[bullId].destEst();
			}
		}
	}

	bool destruirAsteroide = false;

	// Choque asteroides contra satlites
	for (int astId = 0; astId < cantAsteroids; astId++)
	{
		destruirAsteroide = false;
		if (!ast[astId].IsNull())
		{
			// Choque contra Satlite
			for (int satId = 0; satId < cantSatelites; satId++)
			{
				if (!sat[satId].IsNull())
				{
					distCentros = (abs(posXSatelite[satId] - ast[astId].getX())) +
						abs(posYSatelite - ast[astId].getY()) +
						abs(posZSatelite[satId] - ast[astId].getZ());
					if (distCentros < (5))
					{
						// Si choque, destruccin satlite
						sat[satId].setNull();
						cantSatDestruidos++;

						for (int e = 0; e < cantAsteroids; e++){
							if (explo[e].IsNull())
							{
								explo[e].setPosicionExp(Vector3D(ast[astId].getX(),
									ast[astId].getY() - 1.5, ast[astId].getZ()));
								break;
							}
						}

						destruirAsteroide = true;
						break;
					}
				}
			}

			// Destruccin del asteroide por choque o suelo
			if ((destruirAsteroide) || (ast[astId].getY() <= -3))
			{
				ast[astId].setNull();
			}
		}
	}
}

void Colisionador::draw(bool mov)
{
	if (mov)
	{
		Uint32 tActual = SDL_GetTicks();
		if ((tActual - tAsteroide) >= tBetweenAst)
		{
			newAsteroid();
			tAsteroide = SDL_GetTicks();
		}
	}

	colisionar();

	// Dibujamos 
	for (int i = 0; i < cantBullets; i++)
	{
		if (!bull[i].IsNull())
			bull[i].draw(mov);
	}

	for (int i = 0; i < cantAsteroids; i++)
	{
		if (!ast[i].IsNull())
			ast[i].draw(mov);
	}

	for (int i = 0; i < cantSatelites; i++)
	{
		if (!sat[i].IsNull())
			sat[i].draw();
	}

	for (int i = 0; i < 3; i++)
	{
		if (!explo[i].IsNull())
		{
			explo[i].drawExp(mov);
		}
	}
}

int Colisionador::getSatRestantes()
{
	return cantSatelites - cantSatDestruidos;
}

int Colisionador::getPuntaje()
{
	return puntajeAnt + cantAstDestruidos;
}

bool Colisionador::pasarNivel()
{
	return (cantAstDestruidos == 10);
}

void Colisionador::cambiarNivel(int cantAst, int cantSat, bool inicio)
{
	for (int i = 0; i < cantBullets; i++)
	{
		bull[i].~Bullet();
	}

	for (int i = 0; i < cantAsteroids; i++)
	{
		ast[i].~Asteroid();
	}

	for (int i = 0; i < cantAsteroids; i++)
	{
		explo[i].~Explosion();
	}

	for (int i = 0; i < cantSatelites; i++)
	{
		sat[i].~Modelo();
	}

	delete posXSatelite;
	delete posZSatelite;

	cantAsteroids = cantAst;
	cantSatelites = cantSat;

	if (!inicio)
	{
		puntajeAnt += cantAstDestruidos;
	}
	else
	{
		puntajeAnt = 0;
	}
	cantAstDestruidos = 0;

	bull = new Bullet[cantBullets];
	explo = new Explosion[cantAsteroids];
	ast = new Asteroid[cantAsteroids];
	sat = new Modelo[cantSatelites];
	posXSatelite = new float[cantSatelites];
	posZSatelite = new float[cantSatelites];

	cantSatDestruidos = 0;

	if (cantSatelites == 1)
	{
		sat[0].setValores(0.0, posYSatelite, 0.0);
		posXSatelite[0] = 0;
		posZSatelite[0] = 0;
	}
	else
	{
		float angSat = 360.0 / cantSatelites;
		float angAux = 0;
		for (int i = 0; i < cantSatelites; i++)
		{
			posXSatelite[i] = 5.0 * sin(angAux * M_PI / 180);
			posZSatelite[i] = 5.0 * cos(angAux * M_PI / 180);

			sat[i].setValores(posXSatelite[i], posYSatelite, posZSatelite[i]);

			angAux += angSat;
		}
	}

	tAsteroide = SDL_GetTicks();
}

Colisionador::~Colisionador()
{
	for (int i = 0; i < cantBullets; i++)
	{
		bull[i].~Bullet();
	}

	for (int i = 0; i < cantAsteroids; i++)
	{
		ast[i].~Asteroid();
	}

	for (int i = 0; i < cantAsteroids; i++)
	{
		explo[i].~Explosion();
	}

	for (int i = 0; i < cantSatelites; i++)
	{
		sat[i].~Modelo();
	}

	delete posXSatelite;
	delete posZSatelite;
}