#include "..\headers\Explosion.h"

Explosion::Explosion()
{
	isNull = true;
}

Vector3D newSpeed(Vector3D dest)
{
	Vector3D velocidad;
	velocidad.setXYZ(
		(2.0 * ((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0,
		(2.0 * ((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0,
		(2.0 * ((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0);
	velocidad.normalizar();

	return velocidad;
}

void Explosion::setPosicionExp(Vector3D pos)
{
	isNull = false;

	int    i;

	for (i = 0; i < NUM_PARTICLES; i++)
	{
		particulas[i].position = pos;

		particulas[i].color[0] = 0.8;
		particulas[i].color[1] = 0.8;
		particulas[i].color[2] = 0.0;
		particulas[i].color[3] = 1.0;

		particulas[i].speed = newSpeed(particulas[i].speed);
	}

	for (i = 0; i < NUM_DEBRIS; i++)
	{
		debris[i].position = pos;

		debris[i].color[0] = 0.9;
		debris[i].color[1] = 0.2;
		debris[i].color[2] = 0.0;
		debris[i].color[3] = 1.0;

		debris[i].scale[0] = (2.0 *
			((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0;
		debris[i].scale[1] = (2.0 *
			((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0;
		debris[i].scale[2] = (2.0 *
			((GLfloat)rand()) / ((GLfloat)RAND_MAX)) - 1.0;

		debris[i].speed = newSpeed(debris[i].speed);
	}

	fuel = 50;
}

void Explosion::setEstela(Vector3D pos)
{
	isNull = false;

	int    i;

	for (i = 0; i < NUM_PARTICLES; i++)
	{
		particulas[i].position = pos;

		particulas[i].color[0] = 1.0;
		particulas[i].color[1] = 0.5;
		particulas[i].color[2] = 0.3;
		particulas[i].color[3] = 0.0;

		particulas[i].speed = newSpeed(particulas[i].speed);
		particulas[i].position += (particulas[i].speed * 0.4);
	}
	
	fuel = 4;
}

void Explosion::drawExp(bool mov)
{
	if (fuel == 0)
	{
		isNull = true;
	}
	else
	{
		int i;
		glPushMatrix();
		glDisable(GL_LIGHTING);
		glBegin(GL_POINTS);

		for (i = 0; i < NUM_PARTICLES; i++)
		{
			glColor4fv(particulas[i].color);
			glVertex3f(particulas[i].position.getX(),
				particulas[i].position.getY(),
				particulas[i].position.getZ());
		}

		glEnd();
		glPopMatrix();

		glNormal3f(0.0, 0.0, 1.0);

		for (i = 0; i < NUM_DEBRIS; i++)
		{
			glColor4fv(debris[i].color);

			glPushMatrix();

			glTranslatef(debris[i].position.getX(),
				debris[i].position.getY(),
				debris[i].position.getZ());

			glScalef(debris[i].scale[0],
				debris[i].scale[1],
				debris[i].scale[2]);

			glBegin(GL_TRIANGLES);
			glVertex3f(0.0, 0.5, 0.0);
			glVertex3f(-0.25, 0.0, 0.0);
			glVertex3f(0.25, 0.0, 0.0);
			glEnd();

			glPopMatrix();
		}

		glEnable(GL_LIGHTING);
		glEnable(GL_DEPTH_TEST);

		for (i = 0; i < NUM_PARTICLES; i++)
		{
			if (mov)
			{
				particulas[i].position += (particulas[i].speed * 0.2);
				particulas[i].color[0] -= 1.0 / 100.0;
				particulas[i].color[1] -= 1.0 / 50.0;
				particulas[i].color[2] -= 1.0 / 50.0;
			}
			
			if (particulas[i].color[0] < 0.0)
			{
				particulas[i].color[0] = 0.0;
			}

			if (particulas[i].color[1] < 0.0)
			{
				particulas[i].color[1] = 0.0;
			}

			if (particulas[i].color[2] < 0.0)
			{
				particulas[i].color[2] = 0.0;
			}
		}

		for (i = 0; i < NUM_DEBRIS; i++)
		{
			if (mov)
			{
				debris[i].position += (debris[i].speed * 0.1);
			}
		}

		if (mov)
		{
			--fuel;
		}
	}
}

void Explosion::drawEst(Vector3D velBala, bool mov)
{
	if (fuel == 0)
	{
		isNull = true;
	}
	else if (!isNull)
	{
		int i;
		glPushMatrix();
		glDisable(GL_LIGHTING);
		glBegin(GL_POINTS);

		for (i = 0; i < NUM_PARTICLES; i++)
		{
			glColor4fv(particulas[i].color);
			glVertex3f(particulas[i].position.getX() + ((4-fuel)*(velBala.getX()/2)),
				particulas[i].position.getY() + ((4-fuel)*(velBala.getY()/2)),
				particulas[i].position.getZ() + ((4-fuel)*(velBala.getZ()/2)));
		}

		glEnd();
		glPopMatrix();

		glNormal3f(0.0, 0.0, 1.0);

		for (i = 0; i < NUM_PARTICLES; i++)
		{
			if (mov)
			{
				particulas[i].position -= (particulas[i].speed * 0.1);
				particulas[i].color[0] -= 1.0 / 50.0;
				particulas[i].color[1] -= 1.0 / 100.0;
				particulas[i].color[2] -= 1.0 / 50.0;
			}

			if (particulas[i].color[0] < 0.0)
			{
				particulas[i].color[0] = 0.0;
			}

			if (particulas[i].color[1] < 0.0)
			{
				particulas[i].color[1] = 0.0;
			}

			if (particulas[i].color[2] < 0.0)
			{
				particulas[i].color[2] = 0.0;
			}
		}

		for (i = 0; i < NUM_DEBRIS; i++)
		{
			if (mov)
			{
				debris[i].position += (debris[i].speed * 0.1);
			}
		}

		if (mov)
		{
			--fuel;
		}

		glEnable(GL_LIGHTING);
		glEnable(GL_DEPTH_TEST);
	}
}

bool Explosion::IsNull()
{
	return isNull;
}

void Explosion::setNull()
{
	isNull = true;
}

Explosion::~Explosion()
{
}
