#include "..\headers\GeneralSettings.h"
#include "tinyxml2.h"

using namespace tinyxml2;

GeneralSettings* GeneralSettings::instance = NULL;

GeneralSettings::GeneralSettings()
{
	FPS = 25;
	cantidadCanones[0] = 3;
	cantidadCanones[1] = 4;
	cantidadCanones[2] = 5;
	cantidadSatelites[0] = 1;
	cantidadSatelites[1] = 2;
	cantidadSatelites[2] = 3;
	cantidadAsteroids[0] = 6;
	cantidadAsteroids[1] = 7;
	cantidadAsteroids[2] = 8;
	cantidadBullets = 50;
	satelitesRestantes = 4;
	wireframeOn = false;
	facetadoOInterpolado = false;
	texturasOn = true;
	luzActivada = 1;
	nivel = 1;
	velocidad = 2;
}

GeneralSettings* GeneralSettings::Instance()
{
	if (instance == NULL)
	{
		GeneralSettings::instance = new GeneralSettings();
	}

	return instance;
}

void GeneralSettings::loadSettings(){
	XMLDocument doc;
	doc.LoadFile("setting.xml");
	XMLElement* root = doc.RootElement();
	int nivelXML = 0;
	for (XMLElement* level = root->FirstChildElement(); level; level= level->NextSiblingElement()){
		for (XMLElement* setting = level->FirstChildElement(); setting; setting = setting->NextSiblingElement()){
			const char* name = setting->Value();
			const char* value = setting->GetText();
			if (strcmp(name, "cantidadCanones") == 0){
				cantidadCanones[nivelXML] = atoi(value);
			}
			else {
				if (strcmp(name, "cantidadSatelites") == 0){
					cantidadSatelites[nivelXML] = atoi(value);
				}
				else {
					if (strcmp(name, "cantidadAsteroids") == 0){
						cantidadAsteroids[nivelXML] = atoi(value);
					}
				}
			}
		}
		nivelXML++;
	}
}

int GeneralSettings::getLevel()
{
	return nivel;
}

void GeneralSettings::changeLevel()
{
	switch (nivel)
	{
		case 1:
			nivel = 2;
			break;
		case 2:
			nivel = 3;
			break;
	}
}

void GeneralSettings::setLevel()
{
	nivel = 1;
}

int GeneralSettings::getFPS()
{
	return FPS;
}

int GeneralSettings::getCantCanones()
{
	return cantidadCanones[nivel-1];
}

int GeneralSettings::getCantBullets()
{
	return cantidadBullets;
}

int GeneralSettings::getCantAsteroids()
{
	return cantidadAsteroids[nivel - 1];
}

int GeneralSettings::getCantSatelites()
{
	return cantidadSatelites[nivel - 1];
}

int GeneralSettings::getSatelitesRestantes()
{
	return satelitesRestantes;
}

void GeneralSettings::setSatelitesRestantes(int s)
{
	satelitesRestantes = s;
}

bool GeneralSettings::getWireframeOn()
{
	return wireframeOn;
}

void GeneralSettings::changeWirefreameState()
{
	if (wireframeOn)
		wireframeOn = false;
	else
		wireframeOn = true;
}

bool GeneralSettings::getFacetado()
{
	return facetadoOInterpolado;
}

void GeneralSettings::changeFacetadoState()
{
	if (facetadoOInterpolado)
		facetadoOInterpolado = false;
	else
		facetadoOInterpolado = true;
}

bool GeneralSettings::getTexturasOn()
{
	return texturasOn;
}

void GeneralSettings::changeTexturasState()
{
	if (texturasOn)
		texturasOn = false;
	else
		texturasOn = true;
}

int GeneralSettings::getLuzActivada()
{
	return luzActivada;
}

void GeneralSettings::changeLuzActivada()
{
	luzActivada = (luzActivada + 1) % 3;
}

float GeneralSettings::getVelocidad()
{
	return velocidad;
}

void GeneralSettings::upVelocidad()
{
	velocidad += 0.03;
}

void GeneralSettings::downVelocidad()
{
	velocidad -= 0.03;
}

GeneralSettings::~GeneralSettings()
{
}
