#include "..\headers\Vector3D.h"

Vector3D::Vector3D() {
	setXYZ(0.0f, 0.0f, 0.0f);
}

Vector3D::Vector3D(float x, float y, float z) {
	setXYZ(x, y, z);
}

Vector3D Vector3D::operator=(const Vector3D &v) {
	setXYZ(v.x, v.y, v.z);
	return *this;
}

float Vector3D::anguloCon(const Vector3D& v) const {
	return acos((*this)*v/modulo()/v.modulo());
}

float Vector3D::getX() const {
	return x;
}

float Vector3D::getY() const {
	return y;
}

float Vector3D::getZ() const {
	return z;
}

float Vector3D::modulo() const {
	return mod;
}

float Vector3D::moduloAlCuadrado() const {
	return (*this)*(*this);
}

void Vector3D::normalizar() {
	*this = *this/modulo();
}

void Vector3D::setXYZ(float x, float y, float z) {
	this->x = x;
	this->y = y;
	this->z = z;
	mod = sqrt(moduloAlCuadrado());
	A[0] = x;
	A[1] = y;
	A[2] = z;
}

void Vector3D::incrementarX(float dX) {
    setXYZ(x+dX, y, z);
}

void Vector3D::incrementarY(float dY) {
    setXYZ(x, y+dY, z);
}

void Vector3D::incrementarZ(float dZ) {
    setXYZ(x, y, z+dZ);
}

Vector3D Vector3D::simetricoSegun(const Vector3D &n) const {
	Vector3D v = *this;
	return 2*n*(n*v) - v;
}

Vector3D Vector3D::rotar(Vector3D eje, float angulo) {
    float rad = angulo*M_PI/180.0f;

    float cosa = cos(rad);
    float sina = sin(rad);

    float resX = (cosa + (eje.x)*(eje.x)*(1-cosa))*x
            + ((eje.x)*(eje.y)*(1-cosa) - (eje.z)*sina)*y
            + ((eje.x)*(eje.z)*(1-cosa) + (eje.y)*sina)*z;

    float resY = ((eje.y)*(eje.x)*(1-cosa) + (eje.z)*sina)*x
            + (cosa + (eje.y)*(eje.y)*(1-cosa))*y
            + ((eje.y)*(eje.z)*(1-cosa) - (eje.x)*sina)*z;

    float resZ = ((eje.z)*(eje.x)*(1-cosa) - (eje.y)*sina)*x
            + ((eje.z)*(eje.y)*(1-cosa) + (eje.x)*sina)*y
            + (cosa + (eje.z)*(eje.z)*(1-cosa))*z;

    return Vector3D(resX, resY, resZ);
}

float Vector3D::distanciaCon(Vector3D v) {
    return sqrt((x-v.x)*(x-v.x) + (y-v.y)*(y-v.y) + (z-v.z)*(z-v.z));
}

Vector3D Vector3D::operator+() const {
	return *this;
}

Vector3D Vector3D::operator-() const {
	return *this*(-1);
}

Vector3D Vector3D::operator+(const Vector3D &v) const {
	return Vector3D(x + v.x, y + v.y, z + v.z);
}

Vector3D Vector3D::operator+=(const Vector3D &v){
	return *this = *this + v;
}

Vector3D Vector3D::operator-(const Vector3D &v) const {
	return *this + -v;
}

Vector3D Vector3D::operator-=(const Vector3D &v){
	return *this = *this - v;
}

float Vector3D::operator*(const Vector3D &v) const {
	return x*v.x + y*v.y + z*v.z;
}

Vector3D Vector3D::operator/(const Vector3D &v) const {
	return Vector3D(y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x);
}

Vector3D Vector3D::operator/=(const Vector3D &v){
	return *this = *this / v;
}

Vector3D Vector3D::operator*(const float &n) const {
	return Vector3D(x*n, y*n, z*n);
}

Vector3D Vector3D::operator/(const float &n) const {
    if(n==0)
        return *this*0;
	return *this*(1/n);
}

bool Vector3D::operator==(const Vector3D &v) const {
	return x==v.x && y==v.y && z==v.z;
}

bool Vector3D::operator!=(const Vector3D &v) const {
	return !(*this == v);
}

Vector3D operator*(const float &n, const Vector3D &v) {
	return v*n;
}

ostream& operator<<(ostream &o, const Vector3D &v) {
	return o << "(" << v.x << ", " << v.y << ", " << v.z << ")";
}

istream& operator>>(istream &i, Vector3D &v) {
	float x, y, z;
	istream& res = i >> x >> y >> z;
	v.setXYZ(x, y, z);
	return res;
}

float* Vector3D::a_3v() {
    return A;
}

Vector3D::~Vector3D() {

}
