#include "..\headers\Canon.h"

Canon::Canon()
{
	SDL_Surface* img = IMG_LoadPNG_RW(SDL_RWFromFile(img_textura, "r"));
	glGenTextures(1, &textura);
	glBindTexture(GL_TEXTURE_2D, textura);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, img->w, img->h, 0, GL_BGR, GL_UNSIGNED_BYTE, img->pixels);
	SDL_FreeSurface(img);

	anguloX = 0.0;
	anguloY = 0.0;
}

void drawCircle(int numSteps, float hl, float radius)
{
	glBegin(GL_POLYGON);
	for (int ii = 0; ii < numSteps; ii++)
	{
		glNormal3f(0, hl, 0);
		float theta = 2.0f * M_PI * float(ii) / float(numSteps);//get the current angle
		float x = radius * cosf(theta);//calculate the x component
		float y = radius * sinf(theta);//calculate the y component
		glVertex3f(x + 0, -hl, y + 0);//output vertex
	}
	glEnd();
}

void drawAro(int numSteps, float * arrX_1, float* arrX_2, float * arrY_1, float* arrY_2, float hl, float r)
{
	glBegin(GL_QUAD_STRIP);
	int j;
	for (j = 0; j <= numSteps; j++)
	{
		glNormal3f(0, hl, 0);
		glTexCoord2f(0, j * 2 * r*M_PI / numSteps);
		glVertex3f(arrX_1[j], hl, arrY_1[j]);
		glNormal3f(0, hl, 0);
		glTexCoord2f(0.2*r, j * 2 * r*M_PI / numSteps);
		glVertex3f(arrX_2[j], hl, arrY_2[j]);
	}
	glEnd();
}

void drawCylinder(int numSteps, float radius, float hl, float* arrX_1, float* arrY_1)
{
	float a = 0.0f;
	float step = (2 * M_PI) / (float)numSteps;


	glBegin(GL_TRIANGLE_STRIP);
	for (int i = numSteps; i >= 0; i--)
	{
		float x = cos(a) * radius;
		float y = sin(a) * radius;
		arrX_1[numSteps - i] = x;
		arrY_1[numSteps - i] = y;
		glNormal3f(x / radius, 0, y / radius);
		glTexCoord2f(0, i * 2 * radius*M_PI / numSteps);
		glVertex3f(x, hl, y);
		glNormal3f(x / radius, 0, y / radius);
		glTexCoord2f(2 * hl, i * 2 * radius*M_PI / numSteps);
		glVertex3f(x, -hl, y);
		a += step;
	}
	glEnd();
}

void drawHalfSphere(int lats, int longs, GLfloat r, float hl)
{
	int i, j;
	for (i = 0; i <= lats; i++)
	{
		double lat0 = M_PI * (-0.5 + (double)(i - 1) / lats);
		double z0 = sin(lat0);
		double zr0 = cos(lat0);

		double lat1 = M_PI * (-0.5 + (double)i / lats);
		double z1 = sin(lat1);
		double zr1 = cos(lat1);

		glBegin(GL_QUAD_STRIP);
		for (j = 0; j <= longs; j++)
		{
			double lng = 2 * M_PI * (double)(j - 1) / longs;
			double x = cos(lng);
			double y = sin(lng);

			double s1, s2, t;
			s1 = ((double)i) / lats;
			s2 = ((double)i + 1) / lats;
			t = ((double)j) / longs;

			glTexCoord2d(s1, t);
			glNormal3d(x * zr0, z0, y * zr0);
			glVertex3d(r*x * zr0, -r*z0 - hl, r*y * zr0);

			glTexCoord2d(s2, t);
			glNormal3d(x * zr1, z1, y * zr1);
			glVertex3d(r*x * zr1, -r*z1 - hl, r* y * zr1);
		}
		glEnd();
	}
}

void Canon::draw(float X, float Y, float Z)
{
	float length = 2;
	float hl = length * 0.5f;
	float radius = 0.9f;
	int numSteps = 100;
	float arrX_1[100 + 1]; // numSteps + 1
	float arrY_1[100 + 1];
	float arrX_2[100 + 1];
	float arrY_2[100 + 1];

	glBindTexture(GL_TEXTURE_2D, textura);
	glTranslatef(X, Y, Z);
	glPushMatrix();
	radius = 0.9;
	drawCircle(numSteps, hl, radius);//tapa de la semi-esfera
	glTranslatef(0.0, hl, 0.0);
	drawHalfSphere(numSteps, numSteps, radius, length);
	glPopMatrix();

	glPushMatrix();
	radius = 0.4;
	glTranslatef(0.0, -hl, 0);
	glRotatef(anguloX, 0.0, 1.0, 0.0);
	if (X != 0)
	{
		if (X < 0)
			glRotatef(90 - anguloY, Z / X, 0.0, -1.0);
		else
			glRotatef(90 - anguloY, -Z / X, 0.0, 1.0);
	}
	else
		glRotatef(90 - anguloY, -Z, 0.0, 0.0);
	glRotatef(180.0, 1.0, 0.0, 1.0);
	glTranslatef(0.0, -hl, 0);

	drawCylinder(numSteps, radius, hl, arrX_2, arrY_2);//cilindro grande
	radius = 0.5;
	drawCylinder(numSteps, radius, hl, arrX_1, arrY_1);//cilindro chico
	drawAro(numSteps, arrX_1, arrX_2, arrY_1, arrY_2, hl, radius);//aro entre cilindros
	glPopMatrix();
	glTranslatef(-X, -Y, -Z);

	glEnd();
}

void Canon::sumarAngulo(bool X)
{
	if (X)
	{
		if (anguloX < 20)
			anguloX += 1 * GeneralSettings::Instance()->getVelocidad();
	}
	else
	{
		if (anguloY < 45)
			anguloY += 1 * GeneralSettings::Instance()->getVelocidad();
	}
}

void Canon::restarAngulo(bool X)
{
	if (X)
	{
		if (anguloX > -20)
			anguloX -= 1 * GeneralSettings::Instance()->getVelocidad();
	}
	else
	{
		if (anguloY > -0)
			anguloY -= 1 * GeneralSettings::Instance()->getVelocidad();
	}
}

float Canon::getAngulo(bool X)
{
	if (X)
		return anguloX;
	else
		return anguloY;
}

Canon::~Canon()
{
}
