#define _USE_MATH_DEFINES

#include <iostream>
#include "SDL.h"
#include "SDL_opengl.h"
#include "GL\glut.h"
#include "..\headers\GeneralSettings.h"
#include "..\headers\Canon.h"
#include "..\headers\Colisionador.h"
#include "..\headers\Skybox.h"
#include "..\headers\Map.h"
#include "..\headers\MenuScreen.h"
#include "..\headers\Hud.h"
#include "..\headers\Light.h"

#define COMIENZO 1
#define JUGANDO 2
#define PAUSA 3
#define PERDIDO 4

using namespace std;

void init()
{
	if (SDL_EnableKeyRepeat(10, 10)<0)
	{
		cerr << "No se pudo establecer el modo key-repeat: " << SDL_GetError() << endl;
		exit(1);
	}

	glClearColor(0.0, 0.0, 0.0, 1.0);

}

void Init3D()
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	glEnable(GL_NORMALIZE);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0, 800.0 / 600.0, 1.0, 200.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

Canon* can;
float* posXCanon;
float posYCanon = 0.0f;
float* posZCanon;
float angCan;
float ang;
int canSeleccionado;
float x;
float y;
float z;

void initCanones(int cantidadCanones)
{
	can = new Canon[cantidadCanones];
	posXCanon = new float[cantidadCanones];
	posZCanon = new float[cantidadCanones];

	angCan = 360.0 / cantidadCanones;
	ang = 0;
	for (int i = 0; i < cantidadCanones; i++)
	{
		posXCanon[i] = 20 * sin(ang * M_PI / 180);
		posZCanon[i] = 20 * cos(ang * M_PI / 180);

		ang += angCan;
	}
}

void limpiarCanones(int cantidadCanones)
{
	for (int i = 0; i < cantidadCanones; i++)
	{
		can[i].~Canon();	
	}

	delete posXCanon;
	delete posZCanon;
}

void initNivel()
{
	canSeleccionado = 0;
	x = posXCanon[canSeleccionado];
	y = posYCanon;
	z = posZCanon[canSeleccionado];
	ang = 0;
}

int main(int main, char** argv)
{
	SDL_Init(SDL_INIT_EVERYTHING);
	SDL_Surface* screen = SDL_SetVideoMode(800, 600, 32, SDL_SWSURFACE | SDL_OPENGL);

	// Inicializacin juego
	int estado = COMIENZO;
	GeneralSettings* generalSettings = GeneralSettings::Instance();
	generalSettings->loadSettings();
	MenuScreen menuS = MenuScreen();
	bool info = true;
	int result;

	//Inicializacin luz
	Light* light = new Light();
	float lightX = 0.0;
	float lightY = 0.0;
	float lightZ = 0.0;
	float lightR = 1.0;
	float lightG = 1.0;
	float lightB = 1.0;
	light->enable();
	light->setPosX(lightX);
	light->setPosY(lightY);
	light->setPosZ(lightZ);
	light->setColorR(lightR);
	light->setColorG(lightG);
	light->setColorB(lightB);

	//FPS
	int FPS = generalSettings->getFPS();
	Uint32 start;

	// Inicializacin mundo
	Skybox sky = Skybox();
	bool fin = false;
	bool cambiarCan = false;
	bool reDibujar = false;
	bool camSettings = false;
	float anguloX = 0;
	float anguloY = 0;

	// Inicializacin mapa
	Map map = Map();
	
	// Inicializacin caones
	int cantCanones = generalSettings->getCantCanones();
	initCanones(cantCanones);

	//Inicializacin del Hud
	Hud hud = Hud();
	hud.setCantCanones(generalSettings->getCantCanones());

	// Manejo de caones
	initNivel();

	// Inicializacin balas y asteriodes
	Colisionador* colisionador = new Colisionador(generalSettings->getCantBullets(), 
												generalSettings->getCantAsteroids(),
												generalSettings->getCantSatelites());
	Uint32 disparo = SDL_GetTicks();
	Uint32 timeBetween;

	init();
	bool pasarNivel = false;
	SDL_Event evento;
	while (!fin)
	{
		start = SDL_GetTicks();

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();

		switch (estado)
		{
			case COMIENZO:
			{
				gluLookAt(0, 0, 20, 0, 0, 0, 0, 1, 0);
				menuS.draw();
				while ((SDL_PollEvent(&evento)))
					{
						switch (evento.type)
						{
							case SDL_KEYDOWN:
								switch (evento.key.keysym.sym)
								{
									case SDLK_ESCAPE:
										fin = true;
										break;
									case SDLK_RETURN:
										result = menuS.procesarEnter();
										switch (result)
										{
											case 1:
												estado = JUGANDO;
												init();
												break;
											case 2:
												estado = JUGANDO;
												init();
												break;
											case 0:
												fin = true;
												break;
										}
										break;

									case SDLK_DOWN:
										if (!camSettings)
										{
											menuS.siguiente();
											camSettings = true;
										}
										break;
									case SDLK_UP:
										if (!camSettings)
										{
											menuS.anterior();
											camSettings = true;
										}
										break;
									default:
										break;
									}
									break;
								case SDL_KEYUP:
									switch (evento.key.keysym.sym)
									{
									case SDLK_UP:
										if (camSettings)
										{
											camSettings = false;
										}
										break;
									case SDLK_DOWN:
										if (camSettings)
										{
											camSettings = false;
										}
										break;
								}
								break;
							case SDL_QUIT:
								fin = true;
								break;
							default:
								break;
						}
					}
				SDL_GL_SwapBuffers();
				break;
			}
			case JUGANDO:
			{
				Init3D();
				gluLookAt(x * 1.2, y + 2.0, z * 1.2, 0, 0, 0, 0, 1, 0);

				// Wirefreame
				if (generalSettings->getWireframeOn())
					glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
				else
					glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

				// Facetado / Interpolado
				if (generalSettings->getFacetado())
					glShadeModel(GL_FLAT);
				else
					glShadeModel(GL_SMOOTH);

				// Texturas
				if (generalSettings->getTexturasOn())
					glEnable(GL_TEXTURE_2D);
				else
					glDisable(GL_TEXTURE_2D);

				if (generalSettings->getLuzActivada() == 0)
				{
					light->disable();
				}
				else
				{
					light->enable();
				}

				if (!cambiarCan)
				{
					while ((SDL_PollEvent(&evento)))
					{
						switch (evento.type)
						{
						case SDL_KEYDOWN:
							switch (evento.key.keysym.sym)
							{
							case SDLK_v:
								generalSettings->upVelocidad();
								break;
							case SDLK_b:
								generalSettings->downVelocidad();
								break;
							case SDLK_2:
								lightX += 0.2;
								light->setPosX(lightX);
								break;
							case SDLK_1:
								lightX -= 0.2;
								light->setPosX(lightX);
								break;
							case SDLK_4:
								lightY += 0.2;
								light->setPosY(lightY);
								break;
							case SDLK_3:
								lightY -= 0.2;
								light->setPosY(lightY);
								break;
							case SDLK_6:
								lightZ += 0.2;
								light->setPosZ(lightZ);
								break;
							case SDLK_5:
								lightZ -= 0.2;
								light->setPosZ(lightZ);
								break;
							case SDLK_7:
								lightR += 0.01;
								if (lightR > 1.0)
									lightR = 0;
								light->setColorR(lightR);
								break;
							case SDLK_8:
								lightG += 0.01;
								if (lightG > 1.0)
									lightG = 0;
								light->setColorG(lightG);
								break;
							case SDLK_9:
								lightB += 0.01;
								if (lightB > 1.0)
									lightB = 0;
								light->setColorB(lightB);
								break;
							case SDLK_0:
								lightX = 0.0;
								lightY = 0.0;
								lightZ = 0.0;
								lightR = 1.0;
								lightG = 1.0;
								lightB = 1.0;
								light->enable();
								light->setPosX(lightX);
								light->setPosY(lightY);
								light->setPosZ(lightZ);
								light->setColorR(lightR);
								light->setColorG(lightG);
								light->setColorB(lightB);
								break;
							case SDLK_ESCAPE:
								fin = true;
								break;
							case SDLK_q:
								fin = true;
								break;
							case SDLK_LEFT:
								can[canSeleccionado].sumarAngulo(true);
								break;
							case SDLK_RIGHT:
								can[canSeleccionado].restarAngulo(true);
								break;
							case SDLK_UP:
								can[canSeleccionado].sumarAngulo(false);
								break;
							case SDLK_DOWN:
								can[canSeleccionado].restarAngulo(false);
								break;
							case  SDLK_c:
								canSeleccionado = (canSeleccionado + 1) % cantCanones;
								cambiarCan = true;
								break;
							case SDLK_SPACE:
								if (!reDibujar)
								{
									timeBetween = SDL_GetTicks();
									if ((timeBetween - disparo) > 250)
									{
										colisionador->newBullet(posXCanon[canSeleccionado], posYCanon, posZCanon[canSeleccionado], can[canSeleccionado].getAngulo(true), can[canSeleccionado].getAngulo(false));
										reDibujar = true;
										disparo = SDL_GetTicks();
									}
								}
								break;
							case SDLK_w:
								if (!camSettings)
								{
									generalSettings->changeWirefreameState();
									camSettings = true;
								}
								break;
							case SDLK_f:
								if (!camSettings)
								{
									generalSettings->changeFacetadoState();
									camSettings = true;
								}
								break;
							case SDLK_t:
								if (!camSettings)
								{
									generalSettings->changeTexturasState();
									camSettings = true;
								}
								break;
							case SDLK_i:
								if (!camSettings)
								{
									generalSettings->changeLuzActivada();
									camSettings = true;
								}
								break;
							case SDLK_F1:
								if (!camSettings)
								{
									info = !info;
									camSettings = true;
								}
								break;
							default:
								break;
							}
							break;
						case SDL_KEYUP:
							switch (evento.key.keysym.sym)
							{
							case SDLK_p:
								estado = PAUSA;
								glDisable(GL_DEPTH_TEST);
								glColor3f(1.0, 0.0, 0.0);
								break;
							case SDLK_SPACE:
								if (reDibujar)
									reDibujar = false;
								break;
							case SDLK_w:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_f:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_t:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_i:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_F1:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							}
							break;
						case SDL_QUIT:
							fin = true;
							break;
						default:
							break;
						}
					}
				}
				else
				{
					// Giro de la cmara hasta el prximo caon
					x = 20 * sin(ang * M_PI / 180);
					z = 20 * cos(ang * M_PI / 180);

					ang = (int)(ang + (angCan / FPS)) % 360;
					if ((posXCanon[canSeleccionado] == x) && (posZCanon[canSeleccionado] == z))
					{
						cambiarCan = false;
					}
				}

				if ((generalSettings->getLevel() < 3) && (colisionador->pasarNivel()))
				{
					pasarNivel = true;
					limpiarCanones(cantCanones);
					generalSettings->changeLevel();
					cantCanones = generalSettings->getCantCanones();
					initCanones(cantCanones);
					initNivel();
					colisionador->cambiarNivel(generalSettings->getCantAsteroids(), generalSettings->getCantSatelites(), false);
				}
				else
				{
					if (colisionador->getSatRestantes() == 0)
					{
						estado = PERDIDO;
					}
				}

				// Dibujado de la pantalla
				sky.draw();
				map.draw();
				for (int i = 0; i < cantCanones; i++)
				{
					can[i].draw(posXCanon[i], posYCanon, posZCanon[i]);
				}
				colisionador->draw(true);

				hud.setNivel(generalSettings->getLevel());
				hud.setFacetado(generalSettings->getFacetado());
				hud.setFPS(generalSettings->getFPS());
				hud.setLuz(generalSettings->getLuzActivada());
				hud.setTexturas(generalSettings->getTexturasOn());
				hud.setWireFrame(generalSettings->getWireframeOn());
				hud.setCiudadesRestantes(colisionador->getSatRestantes());
				hud.setPuntaje(colisionador->getPuntaje());
				glDisable(GL_LIGHTING);

				if (pasarNivel)
				{
					hud.draw(info, false, generalSettings->getLevel());
				}
				else
				{
					hud.draw(info, false, -1);
				}
				glEnable(GL_LIGHTING);

				SDL_GL_SwapBuffers();

				if (pasarNivel)
				{
					pasarNivel = false;
					SDL_Delay(2000);
				}

				break;
			}
			case PAUSA:
			{
				Init3D();
				gluLookAt(x * 1.2, y + 2.0, z * 1.2, 0, 0, 0, 0, 1, 0);

				// Wirefreame
				if (generalSettings->getWireframeOn())
					glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
				else
					glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

				// Facetado / Interpolado
				if (generalSettings->getFacetado())
					glShadeModel(GL_FLAT);
				else
					glShadeModel(GL_SMOOTH);


				// Texturas
				if (generalSettings->getTexturasOn())
					glEnable(GL_TEXTURE_2D);
				else
					glDisable(GL_TEXTURE_2D);

				if (generalSettings->getLuzActivada() == 0)
				{
					light->disable();
				}
				else
				{
					light->enable();
				}

				if (!cambiarCan)
				{
					while ((SDL_PollEvent(&evento)))
					{
						switch (evento.type)
						{
						case SDL_KEYDOWN:
							switch (evento.key.keysym.sym)
							{
							case SDLK_2:
								lightX += 0.2;
								light->setPosX(lightX);
								break;
							case SDLK_1:
								lightX -= 0.2;
								light->setPosX(lightX);
								break;
							case SDLK_4:
								lightY += 0.2;
								light->setPosY(lightY);
								break;
							case SDLK_3:
								lightY -= 0.2;
								light->setPosY(lightY);
								break;
							case SDLK_6:
								lightZ += 0.2;
								light->setPosZ(lightZ);
								break;
							case SDLK_5:
								lightZ -= 0.2;
								light->setPosZ(lightZ);
								break;
							case SDLK_7:
								lightR += 0.01;
								if (lightR > 1.0)
									lightR = 0;
								light->setColorR(lightR);
								break;
							case SDLK_8:
								lightG += 0.01;
								if (lightG > 1.0)
									lightG = 0;
								light->setColorG(lightG);
								break;
							case SDLK_9:
								lightB += 0.01;
								if (lightB > 1.0)
									lightB = 0;
								light->setColorB(lightB);
								break;
							case SDLK_0:
								lightX = 0.0;
								lightY = 0.0;
								lightZ = 0.0;
								lightR = 1.0;
								lightG = 1.0;
								lightB = 1.0;
								light->enable();
								light->setPosX(lightX);
								light->setPosY(lightY);
								light->setPosZ(lightZ);
								light->setColorR(lightR);
								light->setColorG(lightG);
								light->setColorB(lightB);
								break;
							case SDLK_ESCAPE:
								fin = true;
								break;
							case SDLK_q:
								fin = true;
								break;
							case SDLK_w:
								if (!camSettings)
								{
									generalSettings->changeWirefreameState();
									camSettings = true;
								}
								break;
							case SDLK_f:
								if (!camSettings)
								{
									generalSettings->changeFacetadoState();
									camSettings = true;
								}
								break;
							case SDLK_t:
								if (!camSettings)
								{
									generalSettings->changeTexturasState();
									camSettings = true;
								}
								break;
							case SDLK_i:
								if (!camSettings)
								{
									generalSettings->changeLuzActivada();
									camSettings = true;
								}
								break;
							case SDLK_F1:
								if (!camSettings)
								{
									info = !info;
									camSettings = true;
								}
								break;
							default:
								break;
							}
							break;
						case SDL_KEYUP:
							switch (evento.key.keysym.sym)
							{
							case SDLK_p:
								estado = JUGANDO;
								glDisable(GL_DEPTH_TEST);
								glColor3f(1.0, 0.0, 0.0);
								break;
							case SDLK_SPACE:
								if (reDibujar)
									reDibujar = false;
								break;
							case SDLK_w:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_f:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_t:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_i:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							case SDLK_F1:
								if (camSettings)
								{
									camSettings = false;
								}
								break;
							}
							break;
						case SDL_QUIT:
							fin = true;
							break;
						default:
							break;
						}
					}
				}

				// Dibujado de la pantalla
				sky.draw();
				map.draw();
				for (int i = 0; i < cantCanones; i++)
				{
					can[i].draw(posXCanon[i], posYCanon, posZCanon[i]);
				}
				colisionador->draw(false);

				hud.setNivel(generalSettings->getLevel());
				hud.setFacetado(generalSettings->getFacetado());
				hud.setFPS(generalSettings->getFPS());
				hud.setLuz(generalSettings->getLuzActivada());
				hud.setTexturas(generalSettings->getTexturasOn());
				hud.setWireFrame(generalSettings->getWireframeOn());
				hud.setCiudadesRestantes(colisionador->getSatRestantes());
				hud.setPuntaje(colisionador->getPuntaje());
				glDisable(GL_LIGHTING);
				hud.draw(info, false, -1);
				glEnable(GL_LIGHTING);

				SDL_GL_SwapBuffers();

				break;
			}
			case PERDIDO:
			{
				Init3D();
				gluLookAt(x * 1.2, y + 2.0, z * 1.2, 0, 0, 0, 0, 1, 0);

				while ((SDL_PollEvent(&evento)))
				{
					switch (evento.type)
					{
						case SDL_KEYDOWN:
							switch (evento.key.keysym.sym)
							{
							case SDLK_ESCAPE:
								fin = true;
								break;
							case SDLK_q:
								fin = true;
								break;
							case SDLK_r:
								limpiarCanones(cantCanones);
								generalSettings->setLevel();
								cantCanones = generalSettings->getCantCanones();
								initCanones(cantCanones);
								initNivel();
								colisionador->cambiarNivel(generalSettings->getCantAsteroids(), generalSettings->getCantSatelites(), true);
								estado = JUGANDO;
								break;
							}
							break;
						case SDL_QUIT:
							fin = true;
							break;
						default:
							break;
					}
				}

				sky.draw();
				map.draw();
				for (int i = 0; i < cantCanones; i++)
				{
					can[i].draw(posXCanon[i], posYCanon, posZCanon[i]);
				}
				colisionador->draw(true);

				hud.setFacetado(generalSettings->getFacetado());
				hud.setFPS(generalSettings->getFPS());
				hud.setLuz(generalSettings->getLuzActivada());
				hud.setTexturas(generalSettings->getTexturasOn());
				hud.setWireFrame(generalSettings->getWireframeOn());
				hud.setCiudadesRestantes(colisionador->getSatRestantes());
				hud.setPuntaje(colisionador->getPuntaje());
				glDisable(GL_LIGHTING);
				hud.draw(info, true, -1);
				glEnable(GL_LIGHTING);

				SDL_GL_SwapBuffers();
				break;
			}
		}

		//Regulo los FPS
		int a = SDL_GetTicks() - start;
		if (1000 / FPS > a) {
			SDL_Delay(1000 / FPS - (a));
		}
		/*else {
			if (!cambiarCan)
				FPS = 1000/a;
		}*/
	}

	// Liberacin de los recursos
	SDL_Quit();
	sky.~Skybox();
	for (int i = 0; i < cantCanones; i++)
	{
		can[i].~Canon();
	}
	colisionador->~Colisionador();
	
	return 0;
}
